﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace FlowerShopDatabaseImplement.Implements
{
    public class FlowerStorage : IFlowerStorage
    {
        public List<FlowerViewModel> GetFullList()
        {
            using var context = new FlowerShopDatabase();
            return context.Flowers
             .Include(x => x.Components)
            .ThenInclude(x => x.Component)
            .ToList()
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public List<FlowerViewModel> GetFilteredList(FlowerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.FlowerName))
            {
                return new();
            }
            using var context = new FlowerShopDatabase();
            return context.Flowers
             .Include(x => x.Components)
            .ThenInclude(x => x.Component)
            .Where(x => x.FlowerName.Contains(model.FlowerName))
            .ToList()
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public FlowerViewModel? GetElement(FlowerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.FlowerName) &&
            !model.Id.HasValue)
            {
                return null;
            }
            using var context = new FlowerShopDatabase();
            return context.Flowers
             .Include(x => x.Components)
             .ThenInclude(x => x.Component)
             .FirstOrDefault(x => (!string.IsNullOrEmpty(model.FlowerName) &&
            x.FlowerName == model.FlowerName) ||
            (model.Id.HasValue && x.Id ==
            model.Id))
            ?.GetViewModel;
        }
        public FlowerViewModel? Insert(FlowerBindingModel model)
        {
            using var context = new FlowerShopDatabase();
            var newFlower = Flower.Create(context, model);
            if (newFlower == null)
            {
                return null;
            }
            context.Flowers.Add(newFlower);
            context.SaveChanges();
            return newFlower.GetViewModel;
        }
        public FlowerViewModel? Update(FlowerBindingModel model)
        {
            using var context = new FlowerShopDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var product = context.Flowers.FirstOrDefault(rec =>
                rec.Id == model.Id);
                if (product == null)
                {
                    return null;
                }
                product.Update(model);
                context.SaveChanges();
                product.UpdateComponents(context, model);
                transaction.Commit();
                return product.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
        public FlowerViewModel? Delete(FlowerBindingModel model)
        {
            using var context = new FlowerShopDatabase();
            var element = context.Flowers
            .Include(x => x.Components)
            .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Flowers.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
