﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopDatabaseImplement.Models;

namespace FlowerShopDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new FlowerShopDatabase();
            return context.Orders
            .Select(x => AccessFlowerStorage(x.GetViewModel))
           .ToList();
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue)
            {
                return new();
            }
            using var context = new FlowerShopDatabase();
            if (model.Id.HasValue)
                return context.Orders.Where(x => x.Id == model.Id).Select(x => AccessFlowerStorage(x.GetViewModel)).ToList();
            else
                return context.Orders.Where(x => x.DateCreate >= model.DateFrom).Where(x => x.DateCreate <= model.DateTo).Select(x => AccessFlowerStorage(x.GetViewModel)).ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new FlowerShopDatabase();
            return AccessFlowerStorage(context.Orders.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel);
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new FlowerShopDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return AccessFlowerStorage(newOrder.GetViewModel);
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new FlowerShopDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id ==
           model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return AccessFlowerStorage(order.GetViewModel);
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new FlowerShopDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return AccessFlowerStorage(element.GetViewModel);
            }
            return null;
        }

        public static OrderViewModel AccessFlowerStorage(OrderViewModel model)
        {
            if (model == null)
                return null;
            using var context = new FlowerShopDatabase();
            foreach (var Flower in context.Flowers)
            {
                if (Flower.Id == model.FlowerId)
                {
                    model.FlowerName = Flower.FlowerName;
                    break;
                }
            }
            return model;
        }
    }
}
